﻿'------------------------------------------------------------------------------------------------------
' NOTE 1
'   Requires a reference to System.Design in the project this resides in.
' 
' NOTE 2
'   Since no two Excel files are the same there is no magic connection string. With that known 
'   it is good to have a basic understanding of the anatomy of what makes up a connection string 
'   for connecting to Excel worksheets. The following link explains the basics of what goes into 
'   making up a connection string, pay attention to HDR and IMEX.
'
'   http://www.codeproject.com/Articles/37055/Working-with-MS-Excel-xls-xlsx-Using-MDAC-and-Oled
'
' NOTE 3
'   Component Tray docs
'   http://msdn.microsoft.com/en-us/library/system.windows.forms.design.componenttray.aspx
'------------------------------------------------------------------------------------------------------

Imports System.ComponentModel

Public Enum MixType
    Normal = 0
    AsText = 1
    AsTextExtended = 2
End Enum
Public Enum UseHeader
    Yes
    No
End Enum
Public Enum ExcelProvider
    XLS
    XLSX
End Enum
''' <summary>
''' An experiment to allow novice developers a method 
''' to visually setup an Excel connection string
''' </summary>
''' <remarks></remarks>
Public Class ExcelHelper
    Inherits System.ComponentModel.Component

    <Browsable(True)> _
    <EditorBrowsable(EditorBrowsableState.Always)> _
    <System.ComponentModel.Description("Used to setup the proper provider")> _
    <System.ComponentModel.Category("Behavior")> _
    Public Property Provider As ExcelProvider

    <Browsable(True)> _
    <EditorBrowsable(EditorBrowsableState.Always)> _
    <System.ComponentModel.Category("Behavior")> _
    <System.ComponentModel.Description("Indicates if data in sheets has headers as the first row")> _
    Public Property Headers As UseHeader

    <Browsable(True)> _
    <EditorBrowsable(EditorBrowsableState.Always)> _
    <System.ComponentModel.Category("Behavior")> _
    <System.ComponentModel.Description("IMEX")> _
    Public Property Mode As MixType

    Private mFileName As String

    <System.ComponentModel.Category("Behavior")> _
    <System.ComponentModel.Description("Excel file to open")> _
    <Editor(GetType(ExcelFileNameEditor), GetType(System.Drawing.Design.UITypeEditor))> _
    Public Property FileName() As String
        Get
            Return mFileName
        End Get
        Set(value As String)
            mFileName = value
            If IO.Path.GetExtension(value).ToLower = ".xlsx" Then
                Provider = ExcelProvider.XLSX
            Else
                Provider = ExcelProvider.XLS
            End If
        End Set
    End Property
    <Browsable(True)> _
    <EditorBrowsable(EditorBrowsableState.Always)> _
    <System.ComponentModel.Description("Current connection string for file")> _
    Public ReadOnly Property TheConnectionString As String
        Get
            Return Me.ConnectionString
        End Get
    End Property

    Public Sub New()
    End Sub

    ''' <summary>
    ''' Connection to for OleDb connection
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function ConnectionStringMinimum() As String
        SetVersions(Provider)
        Return <Connection>provider=<%= mProvider %>;data source=<%= FileName %>;Extended Properties="Excel <%= OleExelVersion %>;"</Connection>.Value
    End Function
    Public Function ConnectionString() As String
        SetVersions(Provider)
        Return <Connection>Provider=<%= mProvider %>;Data Source=<%= FileName %>;Extended Properties="Excel <%= OleExelVersion %>;IMEX=<%= CInt(Mode) %>;HDR=<%= Me.Headers.ToString & ";" %>"</Connection>.Value
    End Function
    Private OleExelVersion As String = "8.0"
    Private mProvider As String = "Microsoft.Jet.OLEDB.4.0"

    Private Sub SetVersions(ByVal TheProvider As ExcelProvider)
        If TheProvider = ExcelProvider.XLSX Then
            OleExelVersion = "12.0"
            mProvider = "Microsoft.ACE.OLEDB.12.0"
        Else
            OleExelVersion = "8.0"
            mProvider = "Microsoft.Jet.OLEDB.4.0"
        End If

    End Sub
    Public ReadOnly Property IsReady As Boolean
        Get
            If String.IsNullOrWhiteSpace(Me.FileName) Then
                Return False
            End If
            If IO.File.Exists(Me.FileName) Then
                Return True
            Else
                Return False
            End If
        End Get
    End Property
    ''' <summary>
    ''' Return sheet names in Excel file
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks>
    ''' The order of the sheetnames will not match the order
    ''' they are in within the Excel file. There are ways around
    ''' this but not with OleDb.
    ''' 
    ''' Sheet names will end with $ by default
    ''' </remarks>
    Public Function SheetNames() As List(Of String)
        Dim Names As New List(Of String)

        Using cn As New OleDb.OleDbConnection(ConnectionString)

            cn.Open()

            Dim dt As DataTable = cn.GetSchema("Tables", New String() {Nothing, Nothing, Nothing, "Table"})

            For Each row As DataRow In dt.Rows
                Names.Add(row.Field(Of String)("Table_Name"))
            Next

        End Using

        Return Names

    End Function
    ''' <summary>
    ''' Determine if a sheet name exists in the current Excel file
    ''' </summary>
    ''' <param name="SheetName"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function SheetExists(ByVal SheetName As String) As Boolean

        Dim Names As List(Of String) = SheetNames()

        For Each sheet In Names
            If sheet.ToLower = SheetName.ToLower Then
                Return True
            End If
        Next

        Return False

    End Function
End Class

Class ExcelFileNameEditor
    Inherits System.Windows.Forms.Design.FileNameEditor
    Protected Overrides Sub InitializeDialog(ByVal sender As OpenFileDialog)
        MyBase.InitializeDialog(sender)
        sender.Filter = "Excel 2007|*.xlsx|Excel 2003|*.xls"
        sender.Title = "Select Excel File"
    End Sub
End Class
