﻿''' <summary>
''' Shows how to setup a component tray to setup a connection string
''' to an Excel file. As coded the connection string is setup to read
''' data either with headers or no headers for all sheets.
''' </summary>
''' <remarks></remarks>
Public Class Form1
    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        Console.WriteLine(ExcelHelper1.ConnectionString)
        Console.WriteLine(ExcelHelper2.ConnectionString)
        '
        ' Lets make sure the file exists
        '
        If ExcelHelper1.IsReady Then
            cboSheetNames.DataSource = ExcelHelper1.SheetNames
            txtConnectionString.Text = ExcelHelper1.ConnectionString
        Else
            cmdDataInformation.Enabled = False
            cmdLoad.Enabled = False
            MessageBox.Show("Excel file not available.")
        End If

    End Sub
    ''' <summary>
    ''' Open a sheet selected via the ComboBox and get all columns and rows
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub cmdLoad_Click(sender As Object, e As EventArgs) Handles cmdLoad.Click

        Dim dt As New DataTable

        Using cn As New OleDb.OleDbConnection With
            {
                .ConnectionString = ExcelHelper1.ConnectionString
            }
            Using cmd As New OleDb.OleDbCommand With
                {
                    .Connection = cn,
                    .CommandText = "SELECT * FROM [" & cboSheetNames.Text & "]"}
                cn.Open()
                dt.Load(cmd.ExecuteReader)
            End Using
        End Using

        DataGridView1.DataSource = dt

    End Sub
    ''' <summary>
    ''' Examine names and data types
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks>
    ''' Since as coded all sheets are setup with the first row as data each column
    ''' name is Fn
    ''' </remarks>
    Private Sub cmdDataInformation_Click(sender As Object, e As EventArgs) Handles cmdDataInformation.Click
        If DataGridView1.DataSource IsNot Nothing Then
            Dim dt As DataTable = CType(DataGridView1.DataSource, DataTable)

            Dim f As New frmViewColumnInformation
            Try
                For Each col As DataColumn In dt.Columns
                    f.DataGridView1.Rows.Add(New Object() {col.ColumnName, col.DataType.ToString})
                Next
                For Each col As DataGridViewColumn In f.DataGridView1.Columns
                    col.AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells
                Next
                f.ActiveControl = f.cmdClose
                f.ShowDialog()
            Finally
                f.Dispose()
            End Try
        Else
            MessageBox.Show("Must load a sheet first.")
        End If
    End Sub
    ''' <summary>
    ''' Using OleDbConnectionStringBuilder we can split up the properties of the connection
    ''' and if need be at run time change them.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub cmdBuilder_Click(sender As Object, e As EventArgs) Handles cmdBuilder.Click

        Dim Builder As New OleDb.OleDbConnectionStringBuilder With {.ConnectionString = ExcelHelper1.ConnectionString}
        Dim f As New frmBuilderViewer
        Try
            Dim sb As New System.Text.StringBuilder
            sb.AppendLine("DataSource: " & Builder.DataSource)
            sb.AppendLine("Provider: " & Builder.Provider)
            sb.AppendLine("Extended Properties: " & Builder.Item("Extended Properties").ToString)
            f.TextBox1.Text = sb.ToString
            f.ActiveControl = f.cmdClose
            f.ShowDialog()
        Finally
            f.Dispose()
        End Try
    End Sub



End Class
